(******************************************************************************
 *                             Core-extended                                  *
 *                                                                            *
 * Copyright (C) 2008- Jane Street Holding, LLC                               *
 *    Contact: opensource@janestreet.com                                      *
 *    WWW: http://www.janestreet.com/ocaml                                    *
 *                                                                            *
 *                                                                            *
 * This library is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU Lesser General Public                 *
 * License as published by the Free Software Foundation; either               *
 * version 2 of the License, or (at your option) any later version.           *
 *                                                                            *
 * This library is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this library; if not, write to the Free Software        *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                            *
 ******************************************************************************)

(** Translating English into useful data structures *)

open Core.Std

(** [parse_int s]: Like standard int_of_string, but try to convert the first
    twenty english numbers (eg. "one", "two", ... "twenty")

    [parse_int "thirteen"]
    [- : int = 13]

    [parse_int "5296"]
    [- : int = 5296]

    [parse_int "twenty five"]
    [Exception: (Failure int_of_string).]
*)
val parse_int : string -> int

(** [parse_date s]: Convert a date in plain english (eg. "yesterday," "tomorrow," etc.) to a Date.t

    [parse_date "Yesterday"]
    [parse_date "6 8 2010"]
    [parse_date "4 days hence"]
    [parse_date "24 weekdays ago"]
*)

val parse_date : string -> Date.t
